;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x5c01d381d5b86c81efb7b9bb93934af2 . "./code.cg2js/cg-common-control.fasl")
(#x4389badc7e4a66190b8407e4346c53f1 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xf6f75bc4d5f9de8421084843f57ab03e . "./code.cg2js/cg-yes-no-string.fasl")
(#xc88061936adb7516694603a172983f3f . "./code.cg2js/cg-clipboard.fasl")
(#x3efedf9db81e520edaddfae6caec41d6 . "./code.cg2js/cg-dde-utils.fasl")
(#x5e350dd635b8c74d3c7e2d90845cf4e0 . "./code.cg2js/cg-editable-text.fasl")
(#x5cc3c0d83935151c4ba42d4c77bfe1e4 . "./code.cg2js/cg-player.fasl")
(#xd8a38d0236b972a0075663d6452c2a39 . "./code.cg2js/cg-text-or-combo.fasl")
(#xaf7dec67d17ebc9cb84a4239cec8bfc3 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x3cf779f776010a6ace721a9c8e058621 . "./code.cg2js/cg-graphics-context.fasl")
(#x923e6f5430772bc348dedacae393bb90 . "./code.cg2js/cg-choose-printer.fasl")
(#x715edb15a9c4694139faaf1f661fbac9 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xe187eb4afe00ba70ef4295a6168231bc . "./code.cg2js/cg-edit-in-place.fasl")
(#x77f8d99d4dc22675d173d0516df0b1cc . "./code.cg2js/cg-split-bar.fasl")
(#x2f9cee579e0e76a5eb72e58f33872bc4 . "./code.cg2js/cg-timer.fasl")
(#xf8be2cdca4199abf89c2339b3ee4b401 . "./code.cg2js/cg-progress-indicator.fasl")
(#xbe27d5f86cf0fb83b272157ea63681f6 . "./code.cg2js/cg-object-editor.fasl")
(#xbef56faba7e6c80b7ab269c462b87263 . "./code.cg2js/cg-fill-texture.fasl")
(#x9c4997f98506fccf9bd7804291c5460e . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xa17772511389ba64ba206b2ef2963109 . "./code.cg2js/cg-os-window.fasl")
(#x60bc36d626e0a78436aa398936e495bf . "./code.cg2js/cg-color-dialog.fasl")
(#x1f369b5b720f6d4dd8572eea6e3592c1 . "./code.cg2js/cg-combo-box.fasl")
(#xc77805943e57c7ecbf99424c06e15121 . "./code.cg2js/cg-directory-dialog.fasl")
(#xe55d8fedf0b2ee82670fff221beebc05 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xad26f7e06fc6f596af7b1845ae66d389 . "./code.cg2js/cg-tooltip.fasl")
(#xdc13358db772de0b29c947cae356fa4a . "./code.cg2js/cg-lettered-menu.fasl")
(#xcb0f36ed10c5e7001fbb8c723632edda . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x0ff7ec166bf8ce0a77e47a6fbe2b2ea2 . "./code.cg2js/cg-pixmap.fasl")
(#xc75a9d739d841c5f0e766a8deb78b02e . "./code.cg2js/cg.fasl")
(#xed8497f30a55a1a73a5251af335d9676 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x3b18972b32d2ed3d3d7783710ebb0c95 . "./code.cg2js/cg-template-string.fasl")
(#xd022d7eae61ab390e01f1aadddd98d86 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x7c88217905dd9b5d3fe16df76e51cd29 . "./code.cg2js/cg-check-box.fasl")
(#xf0812a8a629c114fda3014d96b9be286 . "./code.cg2js/cg-toggling-widget.fasl")
(#xe3a971c7a9e9c25881d3158964b27b2b . "./code.cg2js/cg-radio-button.fasl")
(#x0f97378647155bd8836d0ca3f8eea5d3 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x3b50323f87bad9e7bf263374704d7e92 . "./code.cg2js/cg-scaling-stream.fasl")
(#xae81cc684d658ac9b7b556e7f217c2de . "./code.cg2js/cg-multi-picture-button.fasl")
(#x9979248cf055f635bbe4c717b2ebc01f . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x0e6cacdf73d8c7ae1589f7b323bbacc9 . "./code.cg2js/cg-grid-widget.fasl")
(#x45ea3185fb8b1d7ea848d4b9c7e22bb5 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xf1c325c37ddbf6d488878624a572ebbc . "./code.cg2js/cg-item-list.fasl")
(#x4e2ec5232b7a67b1718eaeae4b39357f . "./code.cg2js/cg-paren-matching.fasl")
(#xe2a925fbe9be39bf28c3a9a73297b708 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xd8392b217316b99ba2e80b7835c96273 . "./code.cg2js/cg-html-dialog.fasl")
(#x16f543cdd27ba32d1d68fc1a88f7f61e . "./code.cg2js/cg-pixmap-widget.fasl")
(#x262607164c00de6cbf5203850096c28e . "./code.cg2js/cg-project-window.fasl")
(#x8985969ebb39472aa863d8a644aa687d . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xb7adaeeb4a9456cad26189bc49759d9f . "./code.cg2js/cg-toolbar.fasl")
(#x87ea11d0a6ca043b402f6ee29c094e03 . "./code.cg2js/cg-caret.fasl")
(#xb29892721707e9780029fb2a0c4c2106 . "./code.cg2js/cg-scroll-bar.fasl")
(#x99aa25811d358119dd5b1a795f9f1c50 . "./code.cg2js/cg-web-browser.fasl")
(#xd1e7601c3d0be6a10230dd39239e3a3c . "./code.cg2js/cg-dropping-outline.fasl")
(#xe142caf61b7866dae97c0878d9572d89 . "./code.cg2js/cg-font-dialog.fasl")
(#xf68d005899b0417431c875154a05617c . "./code.cg2js/cg-icon-pixmap.fasl")
(#x6787cc055189f5eead082e6cc1c6d872 . "./code.cg2js/cg-yes-no-list.fasl")
(#x5ad2d3c8b93e4ec1c4b276f6200fb875 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x121997ff2e88e8c74eac714df9554089 . "./code.cg2js/cg-lamp.fasl")
(#x97653aaee01d500e915194d90aa11527 . "./code.cg2js/cg-drawable.fasl")
(#x6f793a575be66f26b66afa96bce27d9f . "./code.cg2js/cg-choice-list.fasl")
(#xce46d16b575bb1b081b80c37ca08b63a . "./code.cg2js/cg-curve.fasl")
(#x6fea4fac7b01b0ddb4847b49c8b15153 . "./code.cg2js/cg-base.fasl")
(#x78653c703a2eeed9ed560d1140979752 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x3bccc9d6c15fbd41832b555263bc4c5d . "./code.cg2js/cg-printing.fasl")
(#x6020820f61817ce161e33bb4f9b00268 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x30e4fd7dd8ef6856728aa5238f8aded8 . "./code.cg2js/cg-hotspot.fasl")
(#xc3fa983060ffe42d85e7f146cc67313f . "./code.cg2js/cg-object-editor-layout.fasl")
(#x852f6531dcb652117bd7cae681f8ff08 . "./code.cg2js/cg-utility-dialog.fasl")
(#xd0739141f265fa4bec87bcece88ddc6f . "./code.cg2js/cg-shortcut-menu.fasl")
(#xdabf4970f49d56b6eb7a7bc27cb5a8d8 . "./code.cg2js/cg-comtab.fasl")
(#x90e9ee16fec5bddb775cdd6428ac19bd . "./code.cg2js/cg-os-widget.fasl")
(#x5ed32bad85c3eda8f74546f10256235f . "./code.cg2js/cg-icon.fasl")
(#x4f16b21bdec692f3854df398c420c849 . "./code.cg2js/cg-dev.fasl")
(#x586d69758ebd50ede1bc966d6dbca8f5 . "./code.cg2js/cg-calendar.fasl")
(#xc901d35fc54d8351b28f40d7ed96356d . "./code.cg2js/cg-group-box.fasl")
(#x6d1e8ddbd983b47a8f74bd70fe7d3dbe . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xbf368de8a9cf02c31cfd7fd8dcfa8eba . "./code.cg2js/cg-bitmap-pane.fasl")
(#x08dda2b0521fd021a789e766bdec667a . "./code.cg2js/cg-drag-and-drop.fasl")
(#xb8de084c5cf8259265a902fcf342399d . "./code.cg2js/cg-nodes.fasl")
(#xdf0e65fff368e8e08186883be87ec425 . "./code.cg2js/cg-intersect.fasl")
(#xaa1211e7f740e1558b18e3830e9cd2d0 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x8485415f05b4673408c71d8c94202f49 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x189d94ad32da183db8ec82c46a806341 . "./code.cg2js/cg-class-support.fasl")
(#x0d2fa8c51d0b9f2c05e992061cde4819 . "./code.cg2js/cg-dialog-item.fasl")
(#x43ba9e3b548e23a3d433a4a4df26589b . "./code.cg2js/cg-plot-widget.fasl")
(#xdc0c39dab24ed95f52686b80958577b5 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x509efb8b422db5edcf23777ec0697889 . "./code.cg2js/cg-up-down-control.fasl")
(#xd5a64fb42bd436bdb6aed614f5a6bf42 . "./code.cg2js/cg-rich-edit.fasl")
(#x15b1095916d815d15eed13357e2213c1 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xe08377a126273628dbb2c8f970330b28 . "./code.cg2js/cg-status-bar.fasl")
(#xbdd4b275b5c742f65613aa6e7ea6e326 . "./code.cg2js/cg-class-grid.fasl")
(#xc8d34e483ba2a62254ca8ed883298a71 . "./code.cg2js/cg-wrap-string.fasl")
(#x6b744fe9fbee4676bf21e82d06e62adc . "./code.cg2js/cg-layout.fasl")
(#x1321914755d57be25a977abad24ee1d8 . "./code.cg2js/cg-lisp-widget.fasl")
(#x3027e8b9746d0f4e21f3648048b7dc45 . "./code.cg2js/cg-button.fasl")
(#xe153466339ad2d7b82192544dcb355a9 . "./code.cg2js/cg-string-dialog.fasl")
(#xdf61ae9401b9654cec1062752c1da707 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x36c15ade9ae254087143083b2682027d . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x822c6def7ea167d43b2f045a76b1ff60 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x19e9bebfacbff2976365bdf0805e7e13 . "./code.cg2js/cg-static-text.fasl")
(#xf89cf78eaa390fb87a2dcda36357f325 . "./code.cg2js/cg-outline.fasl")
(#x0b0fc5863902bfc25fd48dbf8ee1e13c . "./code.cg2js/cg-gesture-emulation.fasl")
(#x0ff5ecef411ffca76b45b25a74378499 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xa6a9af5d63094c58ac79e138b934c36e . "./code.cg2js/cg-picture-widget.fasl")
(#xc12c645b531ca01bc451d24f9a18f1e0 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x95bc52e03ba1ebe2dd6d7c0d626ea769 . "./code.cg2js/cg-chart-widget.fasl")
(#x853f8a3ca1b73f6ebcfb2150538d0318 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x672332ab956191c3a231c303381d6feb . "./code.cg2js/cg-lisp-text.fasl")
(#x5c0ec72c3a7d116bbe46e7729abda09b . "./code.cg2js/cg-palette.fasl")
(#x666795e82c5d89598108f90e38e10497 . "./code.cg2js/cg-property.fasl")
(#x3e64a52c22df69d0000296927b34b4ae . "./code.cg2js/cg-intersect-posbox.fasl")
(#x4ac9479dad1ad0395438b3adef67082c . "./code.cg2js/cg-file-dialog.fasl")
(#xad92c0cbd32d4f4a41638fd9c8b40d14 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x30c28739634e98805e31aa1d060b1d6f . "./code.cg2js/cg-get-pixmap.fasl")
(#x86961bd0654aac9d7723be0b4c4c9ee0 . "./code.cg2js/cg-tray.fasl")
(#xb97862eec524c2a66eaa879ecf92955f . "./code.cg2js/cg-acache.fasl")
(#x13e890b3b2ad0bfb9fdbcb7aec198510 . "./code.cg2js/cg-message-dialog.fasl")
(#x9e4fa4f865f37d52af82c4d7117c6f93 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xd314ad354307193ccab56aa741e73f75 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x7448326dc6258d8c5813b10845a1bbde . "./code.cg2js/cg-tab-control.fasl")
(#x2504d98efa539579d13b90ef9388b1f9 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x9eb16ee61074beba222dad169a6555bc . "./code.cg2js/cg-menu-tooltip.fasl")
(#x2b4f53664619276cad95593b62a7720b . "./code.cg2js/cg-find-string-dialog.fasl")
(#x8f9fbcfc21fb345f066c076de59aadf0 . "./code.cg2js/cg-menu.fasl")
(#xb7b0f939903f1ce6e4522cf89736bcfd . "./code.cg2js/cg-get-position.fasl")
(#x61894afa2f666270899a01198445f86d . "./code.cg2js/cg-trackbar.fasl")
(#x5344fc194fa5e1f51c713aa6c53dae27 . "./code.cg2js/cg-selected-object.fasl")
(#x59ace3da161c822930d8b29de21d02c9 . "./code.cg2js/cg-list-view.fasl")
(#x402e3e9a906f5564b642bc216e03715c . "./code.cg2js/cg-html-widget.fasl")
(#xd52664eca35fa997e38a22928fd1fcb9 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x4f36e004b1af339c3adf7bca9e9b37e6 . "./code.cg2js/cg-text-widget.fasl")
(#xfc2cec0908abd5f8fd0c9833efa65333 . "./code.idejs/ide-base.fasl")
(#xfd482fbd647157f13bcf803bee17be98 . "./code.idejs/ide-process-browser.fasl")
(#x843e18e998376425bf4203a469b730f8 . "./code.idejs/ide-patch-dialog.fasl")
(#x8eaedc4d927a772808046e45b8641bc6 . "./code.idejs/ide-keyshort.fasl")
(#x79004843fc6adaa70da143a1929e20a4 . "./code.idejs/ide-window-list.fasl")
(#x16cff2e173adb02af4c7c4234cea231c . "./code.idejs/ide.fasl")
(#x053b4f1442db3a45f435860d7b17a8ef . "./code.idejs/ide-editor.fasl")
(#x2e94f60ed59abe6a0d49daf603d82adf . "./code.idejs/ide-clipboard.fasl")
(#x9f3f1b9724c41bcce738fa6f93bffe66 . "./code.idejs/ide-trace.fasl")
(#xa82ebee89b3664d73d0dec073ae3297f . "./code.idejs/ide-class-browser.fasl")
(#x297ca00c0bcbd6b647c716b608ff2a7f . "./code.idejs/ide-grapher.fasl")
(#x8ebcddc69a2bc38c1f30c4d1a118cf56 . "./code.idejs/ide-apropos.fasl")
(#xa5328cde9cd255cf2be9aab439ba8e2e . "./code.idejs/ide-navigator.fasl")
(#xb02677a154538149f91dd5a0f0e11e0d . "./code.idejs/ide-find-definitions.fasl")
(#x67fb47feccd0ae652fe1391965dd1344 . "./code.idejs/ide-project.fasl")
(#x77702104c73bffbdb85e68b0628e0adf . "./code.idejs/ide-class-grapher.fasl")
(#x29764d31c643dc4adc6e03e46da23545 . "./code.idejs/ide-acache.fasl")
(#x1d2d79924d734091bfdb463abc28c57a . "./code.idejs/ide-find-in-files.fasl")
(#x9fd251f32506f638dae5a9c0fd2c3305 . "./code.idejs/ide-inspector.fasl")
(#x606ef595715be8f3b42dab4351a4d1b8 . "./code.idejs/ide-intro.fasl")
(#xfeeccb27944ef845aa4633d5e617ee7b . "./code.idejs/ide-package-list.fasl")
(#xaebc6f79b96cd7b154807019ac42efec . "./code.idejs/ide-dev.fasl")
(#xdea0bb4abd18f236e859ef964ff86e53 . "./code.idejs/ide-form.fasl")
(#x4469068b47e063cb9811630bd05af4c2 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
